SystemOrganization addCategory: #'XML-Writer-Core'!

!String methodsFor: '*XML-Writer-Core' stamp: 'monty 4/20/2016 07:47'!
embedInXMLMarkupWriter: aMarkupWriter
	aMarkupWriter writer string: self! !

!String methodsFor: '*XML-Writer-Core' stamp: 'monty 5/6/2017 03:49'!
xmlWriterLocalNameAfterPrefix: aPrefix
	"optimized"
	| prefixSize |

	(prefixSize := aPrefix size) == 0
		ifTrue: [^ self]
		ifFalse: [^ self copyFrom: prefixSize + 2 to: self size]! !

!String methodsFor: '*XML-Writer-Core' stamp: 'monty 5/6/2017 03:49'!
xmlWriterPrefixBeforeLocalName
	"optimized"
	| i |

	i := self size.
	[i == 0]
		whileFalse: [
			((self at: i) == $:)
				ifTrue: [^ (self copyFrom: 1 to: i - 1)].
			i := i - 1].
	^ ''.! !

!StandardOrderedDictionary methodsFor: '*XML-Writer-Core' stamp: 'monty 6/24/2017 19:05'!
writeAttributesOn: aWriter
	self keysDo: [:attributeName |
		aWriter
			writeAttributeName: attributeName
			value: (self at: attributeName ifAbsent: '')]! !

!StandardOrderedDictionary methodsFor: '*XML-Writer-Core' stamp: 'monty 6/24/2017 19:05'!
writeCanonicalAttributesOn: aWriter
	| sortedAttributeNames |

	(sortedAttributeNames := self keys) sort: [:a :b | | prefixOfA prefixOfB |
		prefixOfA := a xmlWriterPrefixBeforeLocalName.
		prefixOfB := b xmlWriterPrefixBeforeLocalName.
		prefixOfA = prefixOfB
			ifTrue: [
				"if the prefixes are the same, no need to lookup the namespace"
				(a xmlWriterLocalNameAfterPrefix: prefixOfA)
					<= (b xmlWriterLocalNameAfterPrefix: prefixOfB)]
			ifFalse: [| uriOfA uriOfB |
				"the namespace URI is the primary key and
				the local name is the secondary key"
				uriOfA :=
					aWriter resolveActiveTagWriterAttributePrefix: prefixOfA.
				uriOfB :=
					aWriter resolveActiveTagWriterAttributePrefix: prefixOfB.
				uriOfA < uriOfB
					or: [
						uriOfA = uriOfB
							and: [
								(a xmlWriterLocalNameAfterPrefix: prefixOfA)
									<= (b xmlWriterLocalNameAfterPrefix: prefixOfB)]]]].

	"use to:do: for fast writing"
	1 to: sortedAttributeNames size do: [:i | | attributeName |
		attributeName := sortedAttributeNames at: i.
		aWriter
			writeAttributeName: attributeName
			value: (self at: attributeName)].! !

!UndefinedObject methodsFor: '*XML-Writer-Core' stamp: 'monty 4/20/2016 07:47'!
embedInXMLMarkupWriter: aMarkupWriter! !

!Collection methodsFor: '*XML-Writer-Core' stamp: 'monty 4/20/2016 07:47'!
embedInXMLMarkupWriter: aMarkupWriter
	self do: [:each | each embedInXMLMarkupWriter: aMarkupWriter]! !

!Character methodsFor: '*XML-Writer-Core' stamp: 'monty 4/20/2016 07:47'!
embedInXMLMarkupWriter: aMarkupWriter
	self asString embedInXMLMarkupWriter: aMarkupWriter! !

!Character methodsFor: '*XML-Writer-Core' stamp: 'monty 5/6/2017 03:49'!
isXMLWriterNameChar
	"Range tests are faster and use less memory than a BitmapCharacterSet, and
	a temp is used because Character in newer SpurVM images has no 'value'
	inst var.
	
	NameStartChar ::=
		':' | [A-Z] | '_' | [a-z] | [#xC0-#xD6] | [#xD8-#xF6] | [#xF8-#x2FF] |
		[#x370-#x37D] | [#x37F-#x1FFF] | [#x200C-#x200D] | [#x2070-#x218F] |
		[#x2C00-#x2FEF] | [#x3001-#xD7FF] | [#xF900-#xFDCF] | [#xFDF0-#xFFFD] |
		[#x10000-#xEFFFF]
	NameChar ::=
		NameStartChar | '-' | '.' | [0-9] | #xB7 | [#x0300-#x036F] |
		[#x203F-#x2040]"

	| tempValue |

	(tempValue := self asciiValue) <= 16r7A "$z asciiValue hex"
		ifTrue: [
			tempValue >= 16r61 "$a asciiValue hex"
				ifTrue: [^ true].

			tempValue >= 16r41 "$A asciiValue hex"
				ifTrue: [
					tempValue <= 16r5A "$Z asciiValue hex"
						ifTrue: [^ true].
					^ tempValue == 16r5F]. "$_ asciiValue hex"

			tempValue >= 16r30 "$0 asciiValue hex"
				ifTrue: [
					tempValue <= 16r39 "$9 asciiValue hex"
						ifTrue: [^ true].
					^ tempValue == 16r3A]. "$: asciiValue hex"

			tempValue == 16r2D "$- asciiValue hex"
				ifTrue: [^ true].
			^ tempValue == 16r2E]. "$. asciiValue hex"

	tempValue <= 16r2040
		ifTrue: [
			tempValue <= 16r36F
				ifTrue: [	
					tempValue <= 16rF6
						ifTrue: [
							tempValue <= 16rD6
								ifFalse: [^ tempValue >= 16rD8].
							tempValue >= 16rC0
								ifTrue: [^ true].
							^ tempValue == 16rB7]
						ifFalse: [
							tempValue <= 16r2FF
								ifTrue: [^ tempValue >= 16rF8].
							^ tempValue >= 16r300]]
				ifFalse: [
					tempValue <= 16r1FFF
						ifTrue: [	
							tempValue <= 16r37D
								ifFalse: [^ tempValue >= 16r37F].
							^ tempValue >= 16r370]
						ifFalse: [
							tempValue <= 16r200D
								ifTrue: [^ tempValue >= 16r200C].
							^ tempValue >= 16r203F]]]
	ifFalse: [
			tempValue <= 16rD7FF
				ifTrue: [
					tempValue >= 16r2C00
						ifTrue: [
							tempValue >= 16r3001
								ifTrue: [^ true].
							^ tempValue <= 16r2FEF]
						ifFalse: [
							tempValue <= 16r218F
								ifFalse: [^ false].
							^ tempValue >= 16r2070]]
				ifFalse: [
					tempValue <= 16rFFFD
						ifTrue: [
							tempValue <= 16rFDCF
								ifTrue: [^ tempValue >= 16rF900].
							^ tempValue >= 16rFDF0]
						ifFalse: [
							tempValue >= 16r10000
								ifFalse: [^ false].
							^ tempValue <= 16rEFFFF]]].! !

!Character methodsFor: '*XML-Writer-Core' stamp: 'monty 5/6/2017 03:49'!
isXMLWriterNameStartChar
	"Range tests are faster and use less memory than a BitmapCharacterSet, and
	a temp is used because Character in newer SpurVM images has no 'value'
	inst var.
	
	NameStartChar ::=
		':' | [A-Z] | '_' | [a-z] | [#xC0-#xD6] | [#xD8-#xF6] | [#xF8-#x2FF] |
		[#x370-#x37D] | [#x37F-#x1FFF] | [#x200C-#x200D] | [#x2070-#x218F] |
		[#x2C00-#x2FEF] | [#x3001-#xD7FF] | [#xF900-#xFDCF] | [#xFDF0-#xFFFD] |
		[#x10000-#xEFFFF]"

	| tempValue |

	(tempValue := self asciiValue) <= 16r7A "$z asciiValue hex"
		ifTrue: [
			tempValue >= 16r61 "$a asciiValue hex"
				ifTrue: [^ true].

			tempValue >= 16r41 "$A asciiValue hex"
				ifTrue: [
					tempValue <= 16r5A "$Z asciiValue hex"
						ifTrue: [^ true].
					^ tempValue == 16r5F]. "$_ asciiValue hex"
	
			^ tempValue == 16r3A]. "$: asciiValue hex"

	tempValue <= 16r200D
		ifTrue: [
			tempValue <= 16r2FF
				ifTrue: [
					tempValue >= 16rD8
						ifTrue: [
							tempValue >= 16rF8
								ifTrue: [^ true].
							^ tempValue <= 16rF6]
						ifFalse: [
							tempValue >= 16rC0
								ifFalse: [^ false].
							^ tempValue <= 16rD6]]
				ifFalse: [
					tempValue >= 16r37F
						ifTrue: [
							tempValue >= 16r200C
								ifTrue: [^ true].
							^ tempValue <= 16r1FFF]
						ifFalse: [
							tempValue <= 16r37D
								ifFalse: [^ false].
							^ tempValue >= 16r370]]]
	ifFalse: [
		tempValue <= 16rD7FF
			ifTrue: [
				tempValue >= 16r2C00
					ifTrue: [
						tempValue >= 16r3001
							ifTrue: [^ true].
						^ tempValue <= 16r2FEF]
					ifFalse: [
						tempValue <= 16r218F
							ifFalse: [^ false].
						^ tempValue >= 16r2070]]
			ifFalse: [
				tempValue >= 16rFDF0
					ifTrue: [
						tempValue >= 16r10000
							ifTrue: [^ tempValue <= 16rEFFFF].
						^ tempValue <= 16rFFFD]
					ifFalse: [
						tempValue <= 16rFDCF
							ifFalse: [^ false].
						^ tempValue >= 16rF900]]].! !

Object subclass: #XMLExternalIDWriter
	instanceVariableNames: 'publicID systemID'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLExternalIDWriter commentStamp: 'monty 11/21/2016 03:56' prior: 0!
This class writes the external IDs for doctype declarations and markup declarations, either as "PUBLIC " followed by the public and system ID literals separated by spaces or "SYSTEM " followed by the system ID literal.!

!XMLExternalIDWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 02:00'!
hasPublicID
	^ self publicID notEmpty! !

!XMLExternalIDWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 02:00'!
hasSystemID
	^ self systemID notEmpty! !

!XMLExternalIDWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 02:00'!
isOmittable
	^ self hasPublicID not
		and: [self hasSystemID not]! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:53'!
publicID
	^ publicID ifNil: [publicID := '']! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:53'!
publicID: aPublicID
	publicID := aPublicID! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:54'!
systemID
	^ systemID ifNil: [systemID := '']! !

!XMLExternalIDWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:54'!
systemID: aSystemID
	systemID := aSystemID! !

!XMLExternalIDWriter methodsFor: 'writing' stamp: 'monty 11/21/2016 02:06'!
writeOn: aWriter
	self hasPublicID
		ifTrue: [self writePublicExternalIDOn: aWriter]
		ifFalse: [
			self hasSystemID
				ifTrue: [self writeSystemExternalIDOn: aWriter]]! !

!XMLExternalIDWriter methodsFor: 'writing' stamp: 'monty 11/21/2016 02:06'!
writePublicExternalIDOn: aWriter
	aWriter
		nextPutAll: 'PUBLIC';
		space;
		writeUnescapedString: self publicID;
		space;
		writeUnescapedString: self systemID! !

!XMLExternalIDWriter methodsFor: 'writing' stamp: 'monty 11/21/2016 02:06'!
writeSystemExternalIDOn: aWriter
	aWriter
		nextPutAll: 'SYSTEM';
		space;
		writeUnescapedString: self systemID! !

XMLExternalIDWriter subclass: #XMLNotationExternalIDWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLNotationExternalIDWriter commentStamp: 'monty 11/21/2016 03:55' prior: 0!
This class writes the external ID of a notaion. It omits empty system ID literals for "PUBLIC" external notation IDs.!

!XMLNotationExternalIDWriter methodsFor: 'writing' stamp: 'monty 4/11/2017 04:48'!
writePublicExternalIDOn: aWriter
	aWriter
		nextPutAll: 'PUBLIC';
		space;
		writeUnescapedString: self publicID.
	"the system ID is optional with notation public IDs"
	self hasSystemID
		ifTrue: [
			aWriter
				space;
				writeUnescapedString: self systemID].! !

Object subclass: #XMLMarkupWriter
	instanceVariableNames: 'writer hasStartedWriting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLMarkupWriter commentStamp: 'JAAyer 7/12/2010 02:41' prior: 0!
This is a base class for markup writers. Subclasses override messages like #writePrologue and #writeEpilogue to write markup using the supplied XMLWriter object.!

XMLMarkupWriter subclass: #XMLCDataWriter
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLCDataWriter commentStamp: 'monty 4/20/2016 07:30' prior: 0!
This class writes <!![CDATA[...]]> sections. If canonical XML is enabled, it writes its contents as a regular XML-escaped string instead.!

!XMLCDataWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:22'!
string
	^ string ifNil: [string := '']! !

!XMLCDataWriter methodsFor: 'accessing' stamp: 'monty 12/1/2014 03:55'!
string: aString
	string := aString! !

!XMLCDataWriter methodsFor: 'private' stamp: 'monty 11/30/2016 02:14'!
writeBody
	self writer isCanonical
		ifTrue: [
			self parent
				ifNotNil: [:parent | parent newStringMarkupBeingWritten].
			self writer writeEscapedContent: self string]
		ifFalse: [self writer nextPutAll: self string]! !

!XMLCDataWriter methodsFor: 'private' stamp: 'monty 11/8/2016 10:45'!
writeEpilogue
	self isCanonical
		ifFalse: [
			self writer
				nextPutAll: ']]>';
				formatAfterCData: self]! !

!XMLCDataWriter methodsFor: 'private' stamp: 'monty 11/8/2016 10:45'!
writePrologue
	self isCanonical
		ifFalse: [
			self writer
				formatBeforeCData: self;
				nextPutAll: '<!![CDATA[']! !

XMLMarkupWriter subclass: #XMLCommentWriter
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLCommentWriter commentStamp: 'monty 4/17/2016 07:12' prior: 0!
This class writes XML <!!-- ... --> comments, but not in DTD internal subsets if canonical XML is enabled.!

!XMLCommentWriter methodsFor: 'testing' stamp: 'monty 12/1/2016 01:29'!
isOmittable
	^ self isCanonical
		and: [self isEmbeddedInDoctypeDeclaration]! !

!XMLCommentWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:22'!
string
	^ string ifNil: [string := '']! !

!XMLCommentWriter methodsFor: 'accessing' stamp: 'monty 12/1/2014 03:56'!
string: aString
	string := aString! !

!XMLCommentWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writeBody
	self isOmittable
		ifFalse: [self writer nextPutAll: self string]! !

!XMLCommentWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writeEpilogue
	self isOmittable
		ifFalse: [
			self writer
				nextPutAll: '-->';
				formatAfterComment: self]! !

!XMLCommentWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:45'!
writePrologue
	self isOmittable
		ifFalse: [
			self writer
				formatBeforeComment: self;
				nextPutAll: '<!!--']! !

XMLMarkupWriter subclass: #XMLEmbeddedMarkupWriter
	instanceVariableNames: 'hasEmbeddedStringMarkup hasEmbeddedMarkupWriters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLEmbeddedMarkupWriter commentStamp: 'monty 6/15/2015 02:43' prior: 0!
This is an abstract class for markup writers that support embedding markup using #with:.!

XMLEmbeddedMarkupWriter subclass: #XMLDoctypeDeclarationWriter
	instanceVariableNames: 'root externalIDWriter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLDoctypeDeclarationWriter commentStamp: 'monty 6/15/2015 02:32' prior: 0!
This class writes <!!DOCTYPE ...> declarations. You can set the root element and the public and system IDs using the messages under "accessing" before specifying any internal subset with #with:.!

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:54'!
externalIDWriter
	^ externalIDWriter ifNil: [externalIDWriter := XMLExternalIDWriter new]! !

!XMLDoctypeDeclarationWriter methodsFor: 'testing' stamp: 'monty 11/9/2016 11:59'!
isDoctypeDeclarationWriter
	^ true! !

!XMLDoctypeDeclarationWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 01:37'!
isExternalIDOmittable
	^ self externalIDWriter isOmittable! !

!XMLDoctypeDeclarationWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 00:49'!
isOmittable
	^ self isCanonical! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID
	^ self externalIDWriter publicID! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
publicID: aPublicID
	self externalIDWriter publicID: aPublicID! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:52'!
root
	^ root ifNil: ['']! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:52'!
root: aRootElementName
	root := aRootElementName! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID
	^ self externalIDWriter systemID! !

!XMLDoctypeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
systemID: aSystemID
	self externalIDWriter systemID: aSystemID! !

!XMLDoctypeDeclarationWriter methodsFor: 'deprecated' stamp: 'monty 6/15/2015 01:10'!
withInternalSubset: aStringOrCollectionOrBlock
	self deprecated: 'use #with: instead'.

	self with: aStringOrCollectionOrBlock.! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writeBody: aStringOrCollectionOrBlock
	self isOmittable
		ifFalse: [super writeBody: aStringOrCollectionOrBlock]! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writeEmbeddedMarkupPrologue
	self isOmittable
		ifFalse: [
			self writer
				space;
				nextPut: $[;
				formatBeforeInternalSubset: self]! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:44'!
writeEpilogue
	self isOmittable
		ifTrue: [^ self].

	self hasEmbeddedMarkup
		ifTrue: [
			self writer
				formatAfterInternalSubset: self;
				nextPut: $]].

	self writer
		nextPut: $>;
		formatAfterDoctypeDeclaration: self.! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:50'!
writeExternalID
	self writer space.
	self externalIDWriter writeOn: self writer.! !

!XMLDoctypeDeclarationWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:50'!
writePrologue
	self isOmittable
		ifTrue: [^ self].

	self writer
		formatBeforeDoctypeDeclaration: self;
		nextPutAll: '<!!DOCTYPE';
		space;
		writeName: self root.

	self isExternalIDOmittable
		ifFalse: [self writeExternalID].! !

!XMLEmbeddedMarkupWriter methodsFor: 'testing' stamp: 'monty 12/1/2014 04:41'!
hasEmbeddedMarkup
	^ hasEmbeddedStringMarkup | hasEmbeddedMarkupWriters! !

!XMLEmbeddedMarkupWriter methodsFor: 'testing' stamp: 'monty 11/30/2014 06:17'!
hasEmbeddedMarkupWriters
	^ hasEmbeddedMarkupWriters! !

!XMLEmbeddedMarkupWriter methodsFor: 'testing' stamp: 'monty 11/30/2014 06:19'!
hasEmbeddedStringMarkup
	^ hasEmbeddedStringMarkup! !

!XMLEmbeddedMarkupWriter methodsFor: 'initialization' stamp: 'monty 9/20/2015 23:25'!
initialize
	super initialize.

	hasEmbeddedStringMarkup := false.
	hasEmbeddedMarkupWriters := false.! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 2/26/2016 09:10'!
newMarkupWriterBeingPushed
	self hasStartedWriting
		ifTrue: [
			self hasEmbeddedMarkup
				ifFalse: [self writeEmbeddedMarkupPrologue].
			hasEmbeddedMarkupWriters := true]
		ifFalse: [self write]! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 2/26/2016 09:10'!
newStringMarkupBeingWritten
	self hasStartedWriting
		ifTrue: [
			self hasEmbeddedMarkup
				ifFalse: [self writeEmbeddedMarkupPrologue].
			hasEmbeddedStringMarkup := true]
		ifFalse: [self write]! !

!XMLEmbeddedMarkupWriter methodsFor: 'writing' stamp: 'monty 2/26/2016 09:12'!
with: aStringOrCollectionOrBlock
	self hasStartedWriting
		ifFalse: [
			self
				startWriting;
				writeBody: aStringOrCollectionOrBlock;
				finishWriting]! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 10/12/2014 19:21'!
writeBody
	self writeBody: nil! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 4/20/2016 07:47'!
writeBody: aStringOrCollectionOrBlock
	aStringOrCollectionOrBlock embedInXMLMarkupWriter: self! !

!XMLEmbeddedMarkupWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:27'!
writeEmbeddedMarkupPrologue! !

XMLEmbeddedMarkupWriter subclass: #XMLTagWriter
	instanceVariableNames: 'name xmlnsDeclarations attributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLTagWriter commentStamp: 'monty 4/17/2016 07:18' prior: 0!
This class writes XML tags. You can specifiy the attributes of the start tag using the #attribute* messages, and you can add xmlns: namespace declarations using the #xmlns* messages.

Empty elements will generate self-closed tags like "<empty/>" unless isCanonical is enabled or outputsSelfClosingTags is disabled.

Redundant namespace declarations won't be generated unless omitsRedundantXMLNSDeclarations is disabled.!

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 14:54'!
attributeAt: aName
	^ self attributeAt: aName ifAbsent: ['']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:38'!
attributeAt: aName ifAbsent: aBlock
	^ self attributes at: aName ifAbsent: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:38'!
attributeAt: aName ifAbsentPut: aBlock
	^ self attributes at: aName ifAbsentPut: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:38'!
attributeAt: aName put: aValue
	^ self attributes at: aName put: aValue! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 11/20/2014 19:48'!
attributes
	^ attributes ifNil: [attributes := OrderPreservingStringDictionary new]! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 6/30/2017 23:04'!
attributes: anAssociationCollection
	attributes := OrderPreservingStringDictionary newFrom: anAssociationCollection! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 9/19/2015 02:13'!
hasAttributes
	^ attributes notNil and: [attributes notEmpty]! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 21:48'!
hasXMLNSDeclarations
	^ xmlnsDeclarations notNil and: [xmlnsDeclarations notEmpty]! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 10/6/2014 23:30'!
includesAttribute: anAttribute
	^ self attributes includesKey: anAttribute! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 21:28'!
includesXMLNS: aPrefix
	^ self xmlnsDeclarations includesKey: aPrefix! !

!XMLTagWriter methodsFor: 'testing' stamp: 'monty 10/12/2014 21:12'!
isTagWriter
	^ true! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:07'!
name
	^ name ifNil: [name := '']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'JAAyer 6/20/2010 16:43'!
name: aName
	name := aName! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 3/24/2015 19:26'!
setAttributes: aDictionary
	attributes := aDictionary! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 3/24/2015 19:26'!
setXMLNSDeclarations: aDictionary 
	xmlnsDeclarations := aDictionary! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 12/1/2014 23:36'!
writeEmbeddedMarkupPrologue
	self writer
		nextPut: $>;
		formatAfterStartTag: self! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 4/6/2018 15:58'!
writeEpilogue
	(self hasEmbeddedMarkup
		or: [self writer isCanonical
			or: [self writer outputsSelfClosingTags not]])
		ifTrue: [
			self hasEmbeddedMarkup
				ifFalse: [
					self writer
						nextPut: $>;
						formatAfterStartTag: self].
			self writer
				formatBeforeEndTag: self;
				nextPut: $<;
				nextPut: $/;
				writeName: self name;
				nextPut: $>;
				formatAfterEndTag: self]
		ifFalse: [
			self writer
				nextPut: $/;
				nextPut: $>;
				formatAfterEndTag: self]! !

!XMLTagWriter methodsFor: 'private' stamp: 'monty 11/30/2016 19:56'!
writePrologue
	self writer
		formatBeforeStartTag: self;
		nextPut: $<;
		writeName: self name.
	xmlnsDeclarations
		ifNotNil: [self writer writeXMLNSDeclarations: xmlnsDeclarations].
	attributes
		ifNotNil: [self writer writeAttributes: attributes].! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 20:59'!
xmlns
	"default namespace"
	^ self xmlnsDeclarations at: ''! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
xmlns: aURI
	"set default namespace"
	self xmlnsDeclarations at: '' put: aURI! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 19:47'!
xmlnsAt: aPrefix
	^ self xmlnsDeclarations at: aPrefix ifAbsent: ['']! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 19:47'!
xmlnsAt: aPrefix ifAbsent: aBlock
	^ self xmlnsDeclarations at: aPrefix ifAbsent: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 9/5/2015 08:42'!
xmlnsAt: aPrefix ifAbsentPut: aBlock
	^ self xmlnsDeclarations at: aPrefix ifAbsentPut: aBlock! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 10/6/2014 19:47'!
xmlnsAt: aPrefix put: aValue
	^ self xmlnsDeclarations at: aPrefix put: aValue! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:55'!
xmlnsDeclarations
	^ xmlnsDeclarations
		ifNil: [xmlnsDeclarations := OrderPreservingStringDictionary new: 5]! !

!XMLTagWriter methodsFor: 'accessing' stamp: 'monty 6/30/2017 23:05'!
xmlnsDeclarations: anAssociationCollection
	"set xmlns decls using a dictionary or array of prefix/uri associations.
	empty prefix is default xmlns."
	xmlnsDeclarations :=
		OrderPreservingStringDictionary newFrom: anAssociationCollection! !

XMLMarkupWriter subclass: #XMLEntityReferenceWriter
	instanceVariableNames: 'name isParameter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLEntityReferenceWriter commentStamp: 'monty 4/17/2016 07:13' prior: 0!
This class writes general and parameter entity references.!

!XMLEntityReferenceWriter methodsFor: 'testing' stamp: 'monty 12/2/2014 00:33'!
isParameter
	^ isParameter ifNil: [isParameter := false]! !

!XMLEntityReferenceWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:32'!
isParameter: aBoolean
	isParameter := aBoolean! !

!XMLEntityReferenceWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:22'!
name
	^ name ifNil: [name := '']! !

!XMLEntityReferenceWriter methodsFor: 'accessing' stamp: 'monty 12/2/2014 00:42'!
name: aName
	name := aName! !

!XMLEntityReferenceWriter methodsFor: 'private' stamp: 'monty 4/9/2017 02:44'!
writeBody
	"avoid #writeName: to avoid formatting"
	self writer nextPutAll: self name! !

!XMLEntityReferenceWriter methodsFor: 'private' stamp: 'monty 5/8/2015 04:16'!
writeEpilogue
	self writer
		nextPut: $;;
		formatAfterEntityReference: self! !

!XMLEntityReferenceWriter methodsFor: 'private' stamp: 'monty 12/2/2014 00:34'!
writePrologue
	self writer
		formatBeforeEntityReference: self;
		nextPut:
			(self isParameter
				ifTrue: [$%]
				ifFalse: [$&])! !

XMLMarkupWriter subclass: #XMLMarkupDeclarationWriter
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLMarkupDeclarationWriter commentStamp: 'monty 4/17/2016 07:16' prior: 0!
This is an abstract class for DTD markup declaration writers.!

XMLMarkupDeclarationWriter subclass: #XMLAttributeDeclarationWriter
	instanceVariableNames: 'attribute type defaultPragma defaultValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLAttributeDeclarationWriter commentStamp: 'monty 4/17/2016 07:15' prior: 0!
This class writes attribute <!!ATTLIST ...> declarations for DTD internal subsets, but not if canonical XML is enabled.!

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:55'!
attribute
	^ attribute ifNil: [attribute := '']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:55'!
attribute: aName
	attribute := aName! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:27'!
declarationType
	^ 'ATTLIST'! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:15'!
defaultPragma
	^ defaultPragma ifNil: [defaultPragma := '']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 2/24/2017 12:31'!
defaultPragma: aDefaultPragma
	defaultPragma :=
		(aDefaultPragma notNil
			and: [aDefaultPragma notEmpty
				and: [aDefaultPragma first == $#]])
			ifTrue: [aDefaultPragma allButFirst]
			ifFalse: [aDefaultPragma]! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:15'!
defaultValue
	^ defaultValue ifNil: [defaultValue := '']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/31/2013 00:17'!
defaultValue: aDefaultValue
	defaultValue := aDefaultValue! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:54'!
element
	^ self name! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 02:54'!
element: aName
	self name: aName! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 13:51'!
type
	^ type ifNil: [type := 'CDATA']! !

!XMLAttributeDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:30'!
type: aType
	type := aType! !

!XMLAttributeDeclarationWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:40'!
writeAttribute
	self writer
		space;
		writeName: self attribute! !

!XMLAttributeDeclarationWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:42'!
writeBody
	self isOmittable
		ifFalse: [
			self
				writeAttribute;
				writeType;
				writeDefaultPragma;
				writeDefaultValue]! !

!XMLAttributeDeclarationWriter methodsFor: 'writing' stamp: 'monty 2/24/2017 12:31'!
writeDefaultPragma
	self defaultPragma notEmpty
		ifTrue: [
			self writer
				space;
				nextPut: $#;
				nextPutAll: self defaultPragma]! !

!XMLAttributeDeclarationWriter methodsFor: 'writing' stamp: 'monty 2/24/2017 12:30'!
writeDefaultValue
	(self defaultPragma isEmpty
		or: [self defaultPragma = 'FIXED'])
		ifTrue: [
			self writer
				space;
				writeEscapedContentString: self defaultValue]! !

!XMLAttributeDeclarationWriter methodsFor: 'private' stamp: 'monty 4/12/2017 01:14'!
writeType
	| typeString typeStringSize isEnumeration nameStart |

	self writer space.
	typeString := self type.
	typeStringSize := typeString size.
	isEnumeration := false.
	1 to: typeStringSize do: [:i | | nextChar |
		nextChar := typeString at: i.
		isEnumeration
			ifTrue: [
				nameStart
					ifNil: [
						nextChar isXMLWriterNameStartChar
							ifTrue: [nameStart := i]
							ifFalse: [self writer nextPut: nextChar]]
					ifNotNil: [
						nextChar isXMLWriterNameChar
							ifFalse: [
								self writer writeName:
									(typeString
										copyFrom: nameStart
										to: i - 1).
								nameStart := nil.
								self writer nextPut: nextChar]]]
			ifFalse: [
				isEnumeration := nextChar == $(.
				self writer nextPut: nextChar]].

	nameStart
		ifNotNil: [
			self writer writeName:
				(typeString
					copyFrom: nameStart
					to: typeStringSize)].! !

XMLMarkupDeclarationWriter subclass: #XMLElementDeclarationWriter
	instanceVariableNames: 'contentModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLElementDeclarationWriter commentStamp: 'monty 4/17/2016 07:15' prior: 0!
This class writes <!!ELEMENT ...> declarations for DTD internal subsets, but not if canonical XML is enabled.!

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 13:52'!
contentModel
	^ contentModel ifNil: [contentModel := '']! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 15:11'!
contentModel: aContentModelString
	contentModel := aContentModelString! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:27'!
declarationType
	^ 'ELEMENT'! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:17'!
element
	^ self name! !

!XMLElementDeclarationWriter methodsFor: 'accessing' stamp: 'monty 12/21/2013 03:17'!
element: aName
	self name: aName! !

!XMLElementDeclarationWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:38'!
writeBody
	self isOmittable
		ifFalse: [self writeContentModel]! !

!XMLElementDeclarationWriter methodsFor: 'private' stamp: 'monty 4/12/2017 01:14'!
writeContentModel
	| contentModelString contentModelStringSize isEnumeration nameStart isNonElementName |

	self writer space.
	contentModelString := self contentModel.
	contentModelStringSize := contentModelString size.
	isEnumeration := false.
	1 to: contentModelStringSize do: [:i | | nextChar |
		nextChar := contentModelString at: i.
		isEnumeration
			ifTrue: [
				nameStart
					ifNil: [
						nextChar isXMLWriterNameStartChar
							ifTrue: [
								nameStart := i.
								isNonElementName := (contentModelString at: i - 1) == $#]
							ifFalse: [self writer nextPut: nextChar]]
					ifNotNil: [
						nextChar isXMLWriterNameChar
							ifFalse: [
								self
									writeContentModelName:
										(contentModelString
											copyFrom: nameStart
											to: i - 1)
									isNonElementName: isNonElementName.
								nameStart := nil.
								self writer nextPut:  nextChar]]]
			ifFalse: [
				isEnumeration := nextChar == $(.
				self writer nextPut: nextChar]].

	nameStart
		ifNotNil: [
			self
				writeContentModelName:
					(contentModelString
						copyFrom: nameStart
						to: contentModelStringSize)
				isNonElementName: isNonElementName].! !

!XMLElementDeclarationWriter methodsFor: 'private' stamp: 'monty 4/12/2017 01:13'!
writeContentModelName: aName isNonElementName: aBoolean
	aBoolean
		ifTrue: [self writer nextPutAll: aName]
		ifFalse: [self writer writeName: aName]! !

XMLMarkupDeclarationWriter subclass: #XMLEntityDeclarationWriter
	instanceVariableNames: 'isParameter replacement externalIDWriter ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLEntityDeclarationWriter commentStamp: 'monty 4/17/2016 07:16' prior: 0!
This class writes general and parameter <!!ENTITY ...> declarations for DTD internal subsets, but not if canonical XML is enabled.!

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/20/2013 23:41'!
declarationType
	^ self isParameter
		ifTrue: ['ENTITY %']
		ifFalse: ['ENTITY']! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:56'!
externalIDWriter
	^ externalIDWriter ifNil: [externalIDWriter := XMLExternalIDWriter new]! !

!XMLEntityDeclarationWriter methodsFor: 'testing' stamp: 'monty 4/9/2017 07:49'!
isExternalIDOmittable
	^ self externalIDWriter isOmittable! !

!XMLEntityDeclarationWriter methodsFor: 'testing' stamp: 'monty 11/13/2013 19:36'!
isParameter
	^ isParameter ifNil: [isParameter := false]! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/13/2013 19:36'!
isParameter: aBoolean
	isParameter := aBoolean! !

!XMLEntityDeclarationWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 00:53'!
isUnparsed
	^ self ndata notEmpty! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 14:45'!
ndata
	^ ndata ifNil: [ndata := '']! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 17:37'!
ndata: aNotationName
	ndata := aNotationName! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID
	^ self externalIDWriter publicID! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
publicID: aPublicID
	self externalIDWriter publicID: aPublicID! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 8/7/2010 14:44'!
replacement
	^ replacement ifNil: [replacement := '']! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 17:35'!
replacement: aReplacement
	replacement := aReplacement! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID
	^ self externalIDWriter systemID! !

!XMLEntityDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
systemID: aSystemID
	self externalIDWriter systemID: aSystemID! !

!XMLEntityDeclarationWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:49'!
writeBody
	self isOmittable
		ifFalse: [
			self isExternalIDOmittable
				ifTrue: [self writeReplacement]
				ifFalse: [
					self writeExternalID.
					self isUnparsed
						ifTrue: [self writeNData]]]! !

!XMLEntityDeclarationWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:47'!
writeExternalID
	self writer space.
	self externalIDWriter writeOn: self writer.! !

!XMLEntityDeclarationWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:46'!
writeNData
	self writer
		space;
		nextPutAll: 'NDATA';
		space;
		writeName: self ndata! !

!XMLEntityDeclarationWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:45'!
writeReplacement
	self writer
		space;
		writeEscapedEntityString: self replacement! !

!XMLMarkupDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:26'!
declarationType
	self subclassResponsibility! !

!XMLMarkupDeclarationWriter methodsFor: 'testing' stamp: 'monty 11/21/2016 00:43'!
isOmittable
	^ self isCanonical! !

!XMLMarkupDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:43'!
name
	^ name ifNil: ['']! !

!XMLMarkupDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 6/20/2010 17:02'!
name: aName
	name := aName! !

!XMLMarkupDeclarationWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:43'!
writeEpilogue
	self isOmittable
		ifFalse: [
			self writer
				nextPut: $>;
				formatAfterDeclaration: self]! !

!XMLMarkupDeclarationWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:39'!
writePrologue
	self isOmittable
		ifFalse: [
			self writer
				formatBeforeDeclaration: self;
				nextPutAll: '<!!';
				nextPutAll: self declarationType;
				space;
				writeName: self name]! !

XMLMarkupDeclarationWriter subclass: #XMLNotationDeclarationWriter
	instanceVariableNames: 'externalIDWriter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLNotationDeclarationWriter commentStamp: 'monty 4/17/2016 07:15' prior: 0!
This class writes <!!NOTATION ...> declarations for DTD internal subsets, but not if canonical XML is enabled.!

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:27'!
declarationType
	^ 'NOTATION'! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/21/2016 02:07'!
externalIDWriter
	^ externalIDWriter ifNil: [externalIDWriter := XMLNotationExternalIDWriter new]! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
publicID
	^ self externalIDWriter publicID! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/22/2015 02:19'!
publicID: aPublicID
	self externalIDWriter publicID: aPublicID! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:20'!
systemID
	^ self externalIDWriter systemID! !

!XMLNotationDeclarationWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:47'!
systemID: aSystemID
	self externalIDWriter systemID: aSystemID! !

!XMLNotationDeclarationWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:47'!
writeBody
	self isOmittable
		ifFalse: [self writeExternalID]! !

!XMLNotationDeclarationWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:47'!
writeExternalID
	self writer space.
	self externalIDWriter writeOn: self writer.! !

!XMLMarkupWriter class methodsFor: 'instance creation' stamp: 'monty 9/20/2015 23:24'!
writer: aWriter
	^ self new writer: aWriter! !

!XMLMarkupWriter methodsFor: 'enumerating' stamp: 'monty 11/30/2016 02:21'!
ancestorsDo: aBlock
	self writer
		ancestorsOfMarkupWriter: self
		do: aBlock! !

!XMLMarkupWriter methodsFor: 'converting' stamp: 'monty 9/19/2015 21:59'!
asString
	^ self writer asString! !

!XMLMarkupWriter methodsFor: 'closing' stamp: 'monty 5/6/2015 16:31'!
close
	self writer close! !

!XMLMarkupWriter methodsFor: 'accessing' stamp: 'monty 5/6/2015 16:31'!
contents
	^ self writer contents! !

!XMLMarkupWriter methodsFor: 'writing' stamp: 'monty 2/26/2016 09:09'!
finishWriting
	self hasEmbeddedMarkupWriters
		ifTrue: [self writer writeUnwrittenMarkupWritersEmbeddedIn: self].
	self writeEpilogue.

	self writer popActiveMarkupWriter.! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 1/14/2014 00:18'!
hasEmbeddedMarkup
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 10/4/2014 04:21'!
hasEmbeddedMarkupWriters
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 12/1/2014 05:19'!
hasEmbeddedStringMarkup
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 2/26/2016 09:11'!
hasStartedWriting
	^ hasStartedWriting! !

!XMLMarkupWriter methodsFor: 'initialization' stamp: 'monty 2/26/2016 09:11'!
initialize
	super initialize.

	hasStartedWriting := false.! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 11/8/2016 10:45'!
isCanonical
	^ self writer isCanonical! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 11/9/2016 11:59'!
isDoctypeDeclarationWriter
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 11/30/2016 02:14'!
isEmbedded
	^ self parent notNil! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 12/1/2016 01:29'!
isEmbeddedInDoctypeDeclaration
	self parent
		ifNil: [^ false]
		ifNotNil: [:parent | ^ parent isDoctypeDeclarationWriter]! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 3/28/2017 04:11'!
isEmbeddedInTagWriter
	self parent
		ifNil: [^ false]
		ifNotNil: [:parent | ^ parent isTagWriter]! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 11/30/2016 02:15'!
isEmbeddedWithStringMarkup
	self parent
		ifNil: [^ false]
		ifNotNil: [:parent | ^ parent hasEmbeddedStringMarkup]! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 10/12/2014 21:11'!
isTagWriter
	^ false! !

!XMLMarkupWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 23:21'!
isUnsafeTagWriter
	^ false! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'monty 1/16/2014 23:00'!
newMarkupWriterBeingPushed
	self write! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'monty 11/30/2014 06:21'!
newStringMarkupBeingWritten
	self write! !

!XMLMarkupWriter methodsFor: 'accessing' stamp: 'monty 11/30/2016 02:21'!
parent
	^ self writer parentOfMarkupWriter: self! !

!XMLMarkupWriter methodsFor: 'printing' stamp: 'JAAyer 7/21/2010 09:26'!
printOn: aStream
	self writer printOn: aStream! !

!XMLMarkupWriter methodsFor: 'writing' stamp: 'monty 2/26/2016 09:11'!
startWriting
	hasStartedWriting := true.
	self writePrologue.! !

!XMLMarkupWriter methodsFor: 'enumerating' stamp: 'monty 11/30/2016 05:07'!
withAncestorsDo: aBlock
	aBlock value: self.
	self ancestorsDo: aBlock.! !

!XMLMarkupWriter methodsFor: 'writing' stamp: 'monty 2/26/2016 09:10'!
write
	self hasStartedWriting
		ifFalse: [
			self
				startWriting;
				writeBody;
				finishWriting]! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'JAAyer 7/26/2010 12:12'!
writeBody! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'JAAyer 6/20/2010 12:08'!
writeEpilogue! !

!XMLMarkupWriter methodsFor: 'private' stamp: 'JAAyer 6/20/2010 12:08'!
writePrologue! !

!XMLMarkupWriter methodsFor: 'accessing' stamp: 'JAAyer 6/18/2010 22:05'!
writer
	^ writer! !

!XMLMarkupWriter methodsFor: 'accessing' stamp: 'monty 9/21/2015 01:01'!
writer: aWriter
	writer := aWriter! !

XMLMarkupWriter subclass: #XMLPIWriter
	instanceVariableNames: 'target data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLPIWriter commentStamp: 'monty 4/17/2016 07:17' prior: 0!
This class writes <?...?> processing instructions, but not in DTD internal subsets if canonical XML is enabled.!

!XMLPIWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:10'!
data
	^ data ifNil: ['']! !

!XMLPIWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 12:39'!
data: aString
	data := aString! !

!XMLPIWriter methodsFor: 'testing' stamp: 'monty 11/8/2016 07:56'!
hasData
	^ self data notEmpty! !

!XMLPIWriter methodsFor: 'testing' stamp: 'monty 4/12/2017 02:03'!
isDataOmittable
	^ self hasData not! !

!XMLPIWriter methodsFor: 'testing' stamp: 'monty 12/1/2016 01:29'!
isOmittable
	^ self isCanonical
		and: [self isEmbeddedInDoctypeDeclaration]! !

!XMLPIWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:10'!
target
	^ target ifNil: ['']! !

!XMLPIWriter methodsFor: 'accessing' stamp: 'monty 3/21/2015 22:50'!
target: aString
	target := aString! !

!XMLPIWriter methodsFor: 'private' stamp: 'monty 4/12/2017 02:03'!
writeBody
	(self isOmittable
		or: [self isDataOmittable])
		ifFalse: [self writeData]! !

!XMLPIWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:37'!
writeData
	| dataString dataStringSize quotedStringStart quotedStringNameStart quote |

	self writer space.
	dataString := self data.
	dataStringSize := dataString size.
	1 to: dataStringSize do: [:i | | nextChar |
		nextChar := dataString at: i.
		quotedStringStart
			ifNil: [
				quotedStringNameStart
					ifNil: [
						nextChar isXMLWriterNameStartChar
							ifTrue: [quotedStringNameStart := i]
							ifFalse: [
								(nextChar == $"
									or: [nextChar == $'])
									ifTrue: [
										quote := nextChar.
										quotedStringStart := i]
									ifFalse: [self writer nextPut: nextChar]]]
					ifNotNil: [
						nextChar isXMLWriterNameChar
							ifFalse: [
								self writer writeStringName:
									(dataString
										copyFrom: quotedStringNameStart
										to: i - 1).
								quotedStringNameStart := nil.
								(nextChar == $"
									or: [nextChar == $'])
									ifTrue: [
										quote := nextChar.
										quotedStringStart := i]
									ifFalse: [self writer nextPut: nextChar]]]]
			ifNotNil: [
				nextChar == quote
					ifTrue: [
						self writer
							writeUnescapedString:
								(dataString
									copyFrom: quotedStringStart + 1
									to: i - 1)
							quotedBy: quote.
						quotedStringStart := nil]]].

	quotedStringStart
		ifNil: [
			quotedStringNameStart
				ifNotNil: [
					self writer writeStringName:
						(dataString
							copyFrom: quotedStringNameStart
							to: dataStringSize)]]
		ifNotNil: [
			"allow unterminated strings, because PIs can contain arbitrary data"
			self writer nextPutAll:
				(dataString
					copyFrom: quotedStringStart
					to: dataStringSize)].! !

!XMLPIWriter methodsFor: 'private' stamp: 'monty 11/21/2016 00:45'!
writeEpilogue
	self isOmittable
		ifFalse: [
			self writer
				nextPutAll: '?>';
				formatAfterPI: self]! !

!XMLPIWriter methodsFor: 'private' stamp: 'monty 4/11/2017 05:01'!
writePrologue
	self isOmittable
		ifFalse: [
			self writer
				formatBeforePI: self;
				nextPutAll: '<?';
				"do not treat PI targets as names to be consistent
				with XMLXMLDeclarationWriter"
				nextPutAll: self target]! !

XMLMarkupWriter subclass: #XMLUnsafeTagWriter
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLUnsafeTagWriter commentStamp: 'monty 9/19/2015 16:15' prior: 0!
This is an abstract class for tag writers that are "unsafe" because they can make non-wellformed XML if they aren't used correctly.!

XMLUnsafeTagWriter subclass: #XMLUnsafeEndTagWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLUnsafeEndTagWriter commentStamp: 'monty 9/19/2015 16:15' prior: 0!
This class is an end tag writer used to close an element started by an XMLUnsafeStartTagWriter.!

!XMLUnsafeEndTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 22:07'!
writeBody
	self writer writeName: self name! !

!XMLUnsafeEndTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 22:07'!
writeEpilogue
	self writer
		nextPut: $>;
		formatAfterEndTag: self! !

!XMLUnsafeEndTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 22:06'!
writePrologue
	self writer
		formatBeforeEndTag: self;
		nextPut: $<;
		nextPut: $/! !

XMLUnsafeTagWriter subclass: #XMLUnsafeStartTagWriter
	instanceVariableNames: 'xmlnsDeclarations attributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLUnsafeStartTagWriter commentStamp: 'monty 9/19/2015 17:05' prior: 0!
This class is a start tag writer. It does not automatically close itself like XMLTagWriter and will make non-wellformed XML if an end tag isn't generated for it.!

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
attributeAt: aName
	^ self attributeAt: aName ifAbsent: ['']! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
attributeAt: aName ifAbsent: aBlock
	^ self attributes at: aName ifAbsent: aBlock! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
attributeAt: aName ifAbsentPut: aBlock
	^ self attributes at: aName ifAbsentPut: aBlock! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
attributeAt: aName put: aValue
	^ self attributes at: aName put: aValue! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
attributes
	^ attributes ifNil: [attributes := OrderPreservingStringDictionary new]! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 6/30/2017 23:05'!
attributes: anAssociationCollection
	attributes := OrderPreservingStringDictionary newFrom: anAssociationCollection! !

!XMLUnsafeStartTagWriter methodsFor: 'testing' stamp: 'monty 9/19/2015 02:13'!
hasAttributes
	^ attributes notNil and: [attributes notEmpty]! !

!XMLUnsafeStartTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 21:48'!
hasXMLNSDeclarations
	^ xmlnsDeclarations notNil and: [xmlnsDeclarations notEmpty]! !

!XMLUnsafeStartTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 21:26'!
includesAttribute: anAttribute
	^ self attributes includesKey: anAttribute! !

!XMLUnsafeStartTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 21:27'!
includesXMLNS: aPrefix
	^ self xmlnsDeclarations includesKey: aPrefix! !

!XMLUnsafeStartTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 21:26'!
setAttributes: aDictionary
	attributes := aDictionary! !

!XMLUnsafeStartTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 21:26'!
setXMLNSDeclarations: aDictionary 
	xmlnsDeclarations := aDictionary! !

!XMLUnsafeStartTagWriter methodsFor: 'private' stamp: 'monty 11/30/2016 19:56'!
writeBody
	self writer writeName: self name.
	xmlnsDeclarations
		ifNotNil: [self writer writeXMLNSDeclarations: xmlnsDeclarations].
	attributes
		ifNotNil: [self writer writeAttributes: attributes].! !

!XMLUnsafeStartTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 22:03'!
writeEpilogue
	self writer
		nextPut: $>;
		formatAfterStartTag: self! !

!XMLUnsafeStartTagWriter methodsFor: 'private' stamp: 'monty 9/18/2015 22:03'!
writePrologue
	self writer
		formatBeforeStartTag: self;
		nextPut: $<! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:27'!
xmlns
	^ self xmlnsDeclarations at: ''! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:27'!
xmlns: aURI
	self xmlnsDeclarations at: '' put: aURI! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
xmlnsAt: aPrefix
	^ self xmlnsDeclarations at: aPrefix ifAbsent: ['']! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
xmlnsAt: aPrefix ifAbsent: aBlock
	^ self xmlnsDeclarations at: aPrefix ifAbsent: aBlock! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
xmlnsAt: aPrefix ifAbsentPut: aBlock
	^ self xmlnsDeclarations at: aPrefix ifAbsentPut: aBlock! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
xmlnsAt: aPrefix put: aValue
	^ self xmlnsDeclarations at: aPrefix put: aValue! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:26'!
xmlnsDeclarations
	^ xmlnsDeclarations
		ifNil: [xmlnsDeclarations := OrderPreservingStringDictionary new: 5]! !

!XMLUnsafeStartTagWriter methodsFor: 'accessing' stamp: 'monty 6/30/2017 23:05'!
xmlnsDeclarations: anAssociationCollection
	"set xmlns decls using a dictionary or array of prefix/uri associations.
	empty prefix is default xmlns."
	xmlnsDeclarations :=
		OrderPreservingStringDictionary newFrom: anAssociationCollection! !

!XMLUnsafeTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 21:20'!
isTagWriter
	^ true! !

!XMLUnsafeTagWriter methodsFor: 'testing' stamp: 'monty 9/18/2015 23:22'!
isUnsafeTagWriter
	^ true! !

!XMLUnsafeTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:07'!
name
	^ name ifNil: [name := '']! !

!XMLUnsafeTagWriter methodsFor: 'accessing' stamp: 'monty 9/18/2015 21:07'!
name: aString
	name := aString! !

XMLMarkupWriter subclass: #XMLXMLDeclarationWriter
	instanceVariableNames: 'encoding version standalone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLXMLDeclarationWriter commentStamp: 'monty 4/17/2016 07:17' prior: 0!
This class writes <?xml ...?> declarations, but not if canonical XML is enabled.!

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'monty 6/15/2015 04:10'!
encoding
	^ encoding ifNil: [encoding := '']! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:13'!
encoding: anEncoding
	encoding := anEncoding! !

!XMLXMLDeclarationWriter methodsFor: 'testing' stamp: 'monty 4/12/2017 01:36'!
hasEncoding
	^ self encoding notEmpty! !

!XMLXMLDeclarationWriter methodsFor: 'testing' stamp: 'monty 4/12/2017 01:34'!
isOmittable
	^ self isCanonical! !

!XMLXMLDeclarationWriter methodsFor: 'testing' stamp: 'monty 4/12/2017 01:41'!
isStandalone
	^ self standalone! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'monty 4/12/2017 01:41'!
isStandalone: aBoolean
	self standalone: aBoolean! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/19/2013 05:43'!
standalone
	^ standalone ifNil: [standalone := false]! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'monty 4/12/2017 01:37'!
standalone: aBooleanOrString
	standalone :=
		aBooleanOrString isString
			ifTrue: [aBooleanOrString = 'yes']
			ifFalse: [aBooleanOrString]! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'monty 11/30/2016 20:20'!
version
	(version isNil
		or: [version isEmpty])
		ifTrue: [version := '1.0'].
	^ version.! !

!XMLXMLDeclarationWriter methodsFor: 'accessing' stamp: 'monty 2/24/2017 12:28'!
version: aVersionNumberOrString
	version :=
		aVersionNumberOrString isNumber
			ifTrue: [aVersionNumberOrString printShowingDecimalPlaces: 1]
			ifFalse: [| string |
				string := (aVersionNumberOrString ifNil: ['1.0']) asString.
				"avoid #first: for portability"
				string
					copyFrom: 1
					to: (string size min: 3)]! !

!XMLXMLDeclarationWriter methodsFor: 'private' stamp: 'monty 4/12/2017 01:34'!
writeBody
	self isOmittable
		ifFalse: [
			self
				writeVersion;
				writeEncoding;
				writeStandalone]! !

!XMLXMLDeclarationWriter methodsFor: 'private' stamp: 'monty 4/12/2017 23:22'!
writeEncoding
	self hasEncoding
		ifTrue: [
			self writer
				writeAttributeName: 'encoding'
				unescapedValue: self encoding]! !

!XMLXMLDeclarationWriter methodsFor: 'private' stamp: 'monty 4/12/2017 01:34'!
writeEpilogue
	self isOmittable
		ifFalse: [
			self writer
				nextPutAll: '?>';
				formatAfterXMLDeclaration: self]! !

!XMLXMLDeclarationWriter methodsFor: 'private' stamp: 'monty 4/12/2017 01:33'!
writePrologue
	self isOmittable
		ifFalse: [
			self writer
				formatBeforeXMLDeclaration: self;
				nextPutAll: '<?xml']! !

!XMLXMLDeclarationWriter methodsFor: 'private' stamp: 'monty 4/12/2017 23:21'!
writeStandalone
	self standalone
		ifTrue: [
			self writer
				writeAttributeName: 'standalone'
				unescapedValue: 'yes']! !

!XMLXMLDeclarationWriter methodsFor: 'private' stamp: 'monty 4/12/2017 23:21'!
writeVersion
	self writer
		writeAttributeName: 'version'
		unescapedValue: self version! !

Object subclass: #XMLWriter
	instanceVariableNames: 'stream streamStartPosition markupWriterStack formatter lineBreak contentEscapes contentStringEscapes isEscaping isCanonical outputsSelfClosingTags omitsRedundantXMLNSDeclarations hadActiveTagWriter'
	classVariableNames: 'CanonicalLineBreak ContentCanonicalEscapes ContentDefaultEscapes ContentNonEntityEscapes ContentStringCanonicalEscapes ContentStringDefaultEscapes ContentStringNonEntityEscapes DefaultLineBreak EntityValueEscapes PlatformSpecificLineBreak SpaceCharacter'
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLWriter commentStamp: 'monty 9/6/2015 15:13' prior: 0!
XMLWriter generates well-formed XML documents using an API similar to Seaside's canvas and tag brushes.

Markup can be created with the messages under "writing." The ones that take no arguments return instances of XMLMarkupWriter subclasses that can be configured with messages like #name: or #attributeAt:put:. Sending them #write or #with: (if they can have embedded markup) causes the markup writer's configuration to be committed and its markup to be written.

For markup writers that can have embedded markup, the argument to #with: can be a string, a block, a collection (of strings and blocks), or nil. A block can optionally take an argument, the XMLWriter. Creating additional markup will cause any unwritten markup from a previous markup writer to be written, as will sending #write, #contents, #printOn:, #asString, or #close.

Here is an example (evaluate with cmd-p):
	| writer |
	writer := XMLWriter new.
	writer
		enablePrettyPrinting;
		xml;
		comment: 'an example'.
	writer tag
		name: 'example:user';
		xmlnsAt: 'example' put: 'urn://example';
		with: [
			writer
				tag: 'username' with: 'jsmith';
				tag: 'real-name' with: 'John Smith';
				tag: 'age' with: '25';
				tag: 'id' with: '101']

If the last message sent isn't a writing message like #with: or #tag:with:, you may need to send #write to force writing any unwritten markup:
	String streamContents: [:stream |
		(XMLWriter on: stream)
			tag: 'first';
			tag: 'second';
			write].

The second tag forces the first to be written, but the second wouldn't be written if #write wasn't sent.

The #writeWith: message evaluates a block and automatically sends #write after, like how #fileNamed:do: automatically sends #close after:
	String streamContents: [:stream |
		(XMLWriter  on: stream)
			writeWith: [:writer |
				writer
					tag: 'first'
					with: [writer tag: 'second'].
				writer tag: 'third']].!

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 11/8/2016 08:03'!
canonicalLineBreak
	^ CanonicalLineBreak! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 11/8/2016 08:03'!
canonicalLineBreak: aCharacterOrString
	CanonicalLineBreak := aCharacterOrString asString! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 11/8/2016 06:14'!
contentCanonicalEscapes
	^ ContentCanonicalEscapes! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 4/12/2017 02:49'!
contentDefaultEscapes
	^ ContentDefaultEscapes! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 11/8/2016 06:15'!
contentNonEntityEscapes
	^ ContentNonEntityEscapes! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 11/8/2016 06:14'!
contentStringCanonicalEscapes
	^ ContentStringCanonicalEscapes! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 4/12/2017 02:49'!
contentStringDefaultEscapes
	^ ContentStringDefaultEscapes! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 11/8/2016 06:15'!
contentStringNonEntityEscapes
	^ ContentStringNonEntityEscapes! !

!XMLWriter class methodsFor: 'private' stamp: 'monty 8/17/2017 06:42'!
copyXMLExtensionMethods
	"
	self copyXMLExtensionMethods
	"
	
	"This generates XMLWriter String and Character extension methods
	from XMLParser String and Character extension methods.
	These could be placed in a shared .mcz they would both depend on,
	but this is simpler and loads faster."
	#(isXMLNameChar isXMLNameStartChar) do: [:each |
		Character
			compile:
				'isXMLWriter',
				((Character sourceCodeAt: each) allButFirst: 'isXML' size)
			classified: #'*XML-Writer-Core'].
	#(xmlPrefixBeforeLocalName xmlLocalNameAfterPrefix:) do: [:each |
		String
			compile:
				'xmlWriter',
				((String sourceCodeAt: each) allButFirst: 'xml' size)
			classified: #'*XML-Writer-Core'].! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 4/17/2016 05:42'!
defaultLineBreak
	^ DefaultLineBreak! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 4/17/2016 05:42'!
defaultLineBreak: aCharacterOrString
	DefaultLineBreak := aCharacterOrString asString! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 4/12/2017 02:48'!
initialize
	"self initialize"

	self
		initializeDefaultLineBreak;
		initializeCanonicalLineBreak;
		initializePlatformSpecificLineBreak;
		initializeContentDefaultEscapes;
		initializeContentCanonicalEscapes;
		initializeContentNonEntityEscapes;
		initializeContentStringDefaultEscapes;
		initializeContentStringCanonicalEscapes;
		initializeContentStringNonEntityEscapes;
		initializeEntityValueEscapes;
		initializeSpaceCharacter.! !

XMLWriter initialize!

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 11/8/2016 08:04'!
initializeCanonicalLineBreak
	self canonicalLineBreak: Character lf! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 4/12/2017 02:49'!
initializeContentCanonicalEscapes
	ContentCanonicalEscapes :=
		ContentDefaultEscapes copy
			at: 16rD put: '&#xD;'; "CR"
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 4/12/2017 02:49'!
initializeContentDefaultEscapes
	"use an array instead of a dictionary for speed, and
	assign after with #yourself to avoid possible race
	conditions when reinitializing the class"
	ContentDefaultEscapes :=
		(Array new: 255)
			at: $& asciiValue put: '&amp;';
			at: $< asciiValue put: '&lt;';
			at: $> asciiValue put: '&gt;'; "required to escape ]]> in PCDATA"
			at: Character cr asciiValue put: #LineBreak;
			at: Character lf asciiValue put: #LineBreak;
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 4/12/2017 02:50'!
initializeContentNonEntityEscapes
	"Like ContentDefaultEscapes, but allows entity references.
	Use an array instead of a dictionary for speed, and assign
	after with #yourself to avoid possible race conditions
	when reinitializing the class."
	ContentNonEntityEscapes :=
		ContentDefaultEscapes copy
			at: $& asciiValue put: nil;
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 4/12/2017 02:49'!
initializeContentStringCanonicalEscapes
	ContentStringCanonicalEscapes :=
		ContentStringDefaultEscapes copy
			at: 16r9 put: '&#x9;'; "tab"
			at: 16rA put: '&#xA;'; "lf"
			at: 16rD put: '&#xD;'; "cr"
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 4/12/2017 02:49'!
initializeContentStringDefaultEscapes
	"use an array instead of a dictionary for speed, and
	assign after with #yourself to avoid possible race
	conditions when reinitializing the class"
	ContentStringDefaultEscapes :=
		(Array new: 255)
			at: $& asciiValue put: '&amp;';
			at: $< asciiValue put: '&lt;'; "< is not allowed in attr values, but > is"
			at: $" asciiValue put: '&quot;';
			at: Character cr asciiValue put: #LineBreak;
			at: Character lf asciiValue put: #LineBreak;
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 4/12/2017 02:50'!
initializeContentStringNonEntityEscapes
	"Like ContentStringDefaultEscapes, but allows entity references.
	Use an array instead of a dictionary for speed, and assign
	after with #yourself to avoid possible race conditions
	when reinitializing the class."
	ContentStringNonEntityEscapes :=
		ContentStringDefaultEscapes copy
			at: $& asciiValue put: nil;
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 4/17/2016 05:43'!
initializeDefaultLineBreak
	self defaultLineBreak: Character cr! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 10/4/2016 21:40'!
initializeEntityValueEscapes
	"use an array instead of a dictionary for speed, and
	assign after with #yourself to avoid possible race
	conditions when reinitializing the class"
	EntityValueEscapes :=
		(Array new: 255)
			at: $" asciiValue put: '&#34;';
			at: $% asciiValue put: '&#37;';
			at: Character cr asciiValue put: #LineBreak;
			at: Character lf asciiValue put: #LineBreak;
			yourself! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 7/5/2016 07:36'!
initializePlatformSpecificLineBreak
	"'Smalltalk os platformName' is used for compatibility with GS and Squeak
	which don't have the OSPlatform classes, and also instead of the shorter
	'Smalltalk platformName' which doesn't work in all Pharos"
	(Smalltalk os platformName asLowercase beginsWith: 'win')
		ifTrue: [self platformSpecificLineBreak: String crlf]
		ifFalse: [self platformSpecificLineBreak: String lf].! !

!XMLWriter class methodsFor: 'class initialization' stamp: 'monty 5/18/2015 07:07'!
initializeSpaceCharacter
	SpaceCharacter := Character space! !

!XMLWriter class methodsFor: 'instance creation' stamp: 'JAAyer 7/27/2010 13:01'!
new
	^ self on: (String new: 512) writeStream! !

!XMLWriter class methodsFor: 'instance creation' stamp: 'monty 5/6/2015 01:29'!
on: aStream
	^ self basicNew initialize on: aStream! !

!XMLWriter class methodsFor: 'instance creation' stamp: 'monty 5/6/2015 01:26'!
on: aStream formatter: aFormatter
	^ (self on: aStream) formatter: aFormatter! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 7/3/2016 02:10'!
platformSpecificLineBreak
	^ PlatformSpecificLineBreak! !

!XMLWriter class methodsFor: 'accessing' stamp: 'monty 7/3/2016 02:10'!
platformSpecificLineBreak: aCharacterOrString
	PlatformSpecificLineBreak := aCharacterOrString asString! !

!XMLWriter class methodsFor: 'writing' stamp: 'monty 10/26/2014 10:17'!
writeWith: aBlock
	^ self new
		writeWith: aBlock;
		contents! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 10/12/2014 06:57'!
% aParameterEntityReference
	self paramEntityRef: aParameterEntityReference! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 10/12/2014 06:57'!
& anEntityReference
	self entityRef: anEntityReference! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 4/17/2016 05:56'!
activeMarkupWriter
	| markupWriterStackSize |

	"optimized to avoid at:ifAbsent:/ifNotEmpty: closure"
	^ (markupWriterStackSize := markupWriterStack size) > 0
		ifTrue: [markupWriterStack at: markupWriterStackSize]
		ifFalse: [nil]! !

!XMLWriter methodsFor: 'enumerating' stamp: 'monty 11/30/2016 05:08'!
activeMarkupWriterAncestorsDo: aBlock
	self markupWriterStack size - 1 to: 1 by: -1 do: [:i |
		 aBlock value: (self markupWriterStack at: i)]! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 11/30/2016 02:12'!
activeMarkupWriterParent
	| markupWriterStackSize |

	(markupWriterStackSize := self markupWriterStack size) >= 2
		ifTrue: [^ self markupWriterStack at: markupWriterStackSize - 1]
		ifFalse: [^ nil]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/13/2016 21:48'!
activeMarkupWriterWhileChangingDo: aBlock
	"evaluate aBlock for the active (top) markup writer until there isn't
	one or it's the same (still active) writer after evaluating aBlock"

	| markupWriterStackSize activeMarkupWriter previousActiveMarkupWriter |

	"inlined from activeMarkupWriter for speed"
	[(markupWriterStackSize := markupWriterStack size) = 0
		or: [
			(activeMarkupWriter := markupWriterStack at: markupWriterStackSize) ==
				previousActiveMarkupWriter]]
		whileFalse: [
			aBlock value: activeMarkupWriter.
			previousActiveMarkupWriter := activeMarkupWriter]! !

!XMLWriter methodsFor: 'enumerating' stamp: 'monty 11/30/2016 05:06'!
activeMarkupWriterWithAncestorsDo: aBlock
	self markupWriterStack reverseDo: aBlock! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 11/30/2016 05:06'!
activeTagWriter
	self activeMarkupWriterWithAncestorsDo: [:each |
		each isTagWriter
			ifTrue: [^ each]].
	^ nil.! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/30/2016 23:25'!
ancestorsOfMarkupWriter: aMarkupWriter do: aBlock
	| enumeratingAncestors |

	enumeratingAncestors := false.
	self markupWriterStack size to: 1 by: -1 do: [:i |
		enumeratingAncestors
			ifTrue: [aBlock value: (self markupWriterStack at: i)]
			ifFalse: [
				enumeratingAncestors :=
					(self markupWriterStack at: i) == aMarkupWriter]].! !

!XMLWriter methodsFor: 'converting' stamp: 'monty 4/20/2016 07:57'!
asString
	"returns the full, untruncated output, unlike #printString, which may
	truncate"

	^ self contents! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 7/4/2016 06:30'!
atBeginning
	"On GS streams can be 0-based or 1-based, so saving the initial position
	to test the current position works for both types and also handles streams
	that were already written to when the XMLWriter was created"

	^ stream position = streamStartPosition! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
attributeDeclaration
	^ self pushNewMarkupWriter: XMLAttributeDeclarationWriter! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 4/12/2017 02:38'!
canonicalLineBreak
	^ CanonicalLineBreak! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 3/7/2017 13:13'!
canonicalWriterFormatterClass
	^ XMLCanonicalWriterFormatter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/1/2014 04:05'!
cdata
	^ self pushNewMarkupWriter: XMLCDataWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/1/2014 04:05'!
cdata: aString
	self cdata
		string: aString;
		write! !

!XMLWriter methodsFor: 'closing' stamp: 'monty 5/6/2015 16:38'!
close
	self write.
	stream close.! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/1/2014 04:04'!
comment
	^ self pushNewMarkupWriter: XMLCommentWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/1/2014 04:04'!
comment: aString
	self comment
		string: aString;
		write! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 4/12/2017 02:51'!
contentCanonicalEscapes
	^ ContentCanonicalEscapes! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 4/12/2017 02:49'!
contentDefaultEscapes
	^ ContentDefaultEscapes! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/7/2016 03:35'!
contentEscapes
	^ contentEscapes! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/7/2016 03:35'!
contentEscapes: aContentEscapesTable
	contentEscapes := aContentEscapesTable! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 4/12/2017 02:42'!
contentNonEntityEscapes
	^ ContentNonEntityEscapes! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 4/12/2017 02:51'!
contentStringCanonicalEscapes
	^ ContentStringCanonicalEscapes! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 4/12/2017 02:49'!
contentStringDefaultEscapes
	^ ContentStringDefaultEscapes! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/7/2016 03:35'!
contentStringEscapes
	^ contentStringEscapes! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/7/2016 03:35'!
contentStringEscapes: aContentStringEscapesTable
	contentStringEscapes := aContentStringEscapesTable! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 4/12/2017 02:42'!
contentStringNonEntityEscapes
	^ ContentStringNonEntityEscapes! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 4/20/2016 07:57'!
contents
	"returns the full, untruncated output, unlike #printString, which may
	truncate"

	self write.
	^ stream contents.! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 4/12/2017 02:37'!
defaultLineBreak
	^ DefaultLineBreak! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 11/9/2016 12:34'!
doctype
	"an abbreviation"

	^ self doctypeDeclaration! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 11/9/2016 12:34'!
doctypeDeclaration
	^ self pushNewMarkupWriter: XMLDoctypeDeclarationWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
elementDeclaration
	^ self pushNewMarkupWriter: XMLElementDeclarationWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 4/12/2017 02:40'!
enablePlatformSpecificLineBreak
	"use the correct line break (LF or CRLF) for this platform when writing"

	self lineBreak: self platformSpecificLineBreak! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 3/7/2017 13:15'!
enablePrettyPrinting
	"enables 'pretty printing' whitespace formatting to make nicer output"

	self formatter: self prettyPrintingWriterFormatterClass new! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 3/28/2017 05:00'!
enablePrologPrettyPrinting
	"enables 'pretty printing' whitespace formatting, but only for the prolog"

	self formatter: self prologPrettyPrintingWriterFormatterClass new! !

!XMLWriter methodsFor: 'writing - unsafe' stamp: 'monty 12/21/2015 21:49'!
endTag
	"Returns a new XMLUnsafeEndTagWriter for writing an end tag to close an element
	started by a #startTag* message.

	Use the #tag* messages instead if possible."

	^ self pushNewMarkupWriter: XMLUnsafeEndTagWriter! !

!XMLWriter methodsFor: 'writing - unsafe' stamp: 'monty 12/21/2015 21:49'!
endTag: aName
	"Returns a new XMLUnsafeEndTagWriter for writing an end tag with the specified
	name to close an element started by a #startTag* message.

	Use the #tag* messages instead if possible."

	^ self endTag name: aName! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
entityDeclaration
	^ self pushNewMarkupWriter: XMLEntityDeclarationWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 5/8/2015 04:50'!
entityRef
	^ self pushNewMarkupWriter: XMLEntityReferenceWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 7/3/2016 23:59'!
entityRef: aName
	"output a general entity reference like &name;"

	self entityRef
		name: aName;
		write! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 4/12/2017 02:55'!
entityValueEscapes
	^ EntityValueEscapes! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/8/2016 06:29'!
escapesContentEntityReferences
	^ self isEscaping
		and: [(contentEscapes at: $& asciiValue) notNil
			and: [(contentStringEscapes at: $& asciiValue) notNil]]! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 4/12/2017 02:49'!
escapesContentEntityReferences: aBoolean
	self escapesContentEntityReferences = aBoolean
		ifTrue: [^ self].
	aBoolean
		ifTrue: [
			self
				isEscaping: true;
				contentEscapes: self contentDefaultEscapes;
				contentStringEscapes: self contentStringDefaultEscapes]
		ifFalse: [
			self
				contentEscapes: self contentNonEntityEscapes;
				contentStringEscapes: self contentStringNonEntityEscapes]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 9/20/2015 02:54'!
formatAfterCData: aCDataWriter
	formatter
		ifNotNil: [formatter formatAfterCData: aCDataWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:10'!
formatAfterComment: aCommentWriter
	formatter
		ifNotNil: [formatter formatAfterComment: aCommentWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:14'!
formatAfterDeclaration: aDeclarationWriter
	formatter
		ifNotNil: [formatter formatAfterDeclaration: aDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:45'!
formatAfterDoctypeDeclaration: aDoctypeDeclarationWriter
	formatter
		ifNotNil: [formatter formatAfterDoctypeDeclaration: aDoctypeDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:08'!
formatAfterEndTag: aTagWriter
	formatter
		ifNotNil: [formatter formatAfterEndTag: aTagWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:15'!
formatAfterEntityReference: anEntityReferenceWriter
	formatter
		ifNotNil: [formatter formatAfterEntityReference: anEntityReferenceWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:42'!
formatAfterInternalSubset: aDoctypeDeclarationWriter
	formatter
		ifNotNil: [formatter formatAfterInternalSubset: aDoctypeDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:11'!
formatAfterPI: aPIWriter
	formatter
		ifNotNil: [formatter formatAfterPI: aPIWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:34'!
formatAfterStartTag: aTagWriter
	formatter
		ifNotNil: [formatter formatAfterStartTag: aTagWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:10'!
formatAfterXMLDeclaration: anXMLDeclarationWriter
	formatter
		ifNotNil: [formatter formatAfterXMLDeclaration: anXMLDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 9/20/2015 02:54'!
formatBeforeCData: aCDataWriter
	formatter
		ifNotNil: [formatter formatBeforeCData: aCDataWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:39'!
formatBeforeComment: aCommentWriter
	formatter
		ifNotNil: [formatter formatBeforeComment: aCommentWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:39'!
formatBeforeDeclaration: aDeclarationWriter
	formatter
		ifNotNil: [formatter formatBeforeDeclaration: aDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:44'!
formatBeforeDoctypeDeclaration: aDoctypeDeclarationWriter
	formatter
		ifNotNil: [formatter formatBeforeDoctypeDeclaration: aDoctypeDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:34'!
formatBeforeEndTag: aTagWriter
	formatter
		ifNotNil: [formatter formatBeforeEndTag: aTagWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/2/2014 05:41'!
formatBeforeEntityReference: anEntityReferenceWriter
	formatter
		ifNotNil: [formatter formatBeforeEntityReference: anEntityReferenceWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:46'!
formatBeforeInternalSubset: aDoctypeDeclarationWriter
	formatter
		ifNotNil: [formatter formatBeforeInternalSubset: aDoctypeDeclarationWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:12'!
formatBeforePI: aPIWriter
	formatter
		ifNotNil: [formatter formatBeforePI: aPIWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:34'!
formatBeforeStartTag: aTagWriter
	formatter
		ifNotNil: [formatter formatBeforeStartTag: aTagWriter]! !

!XMLWriter methodsFor: 'formatting' stamp: 'monty 3/24/2015 19:34'!
formatBeforeXMLDeclaration: anXMLDeclarationWriter
	formatter
		ifNotNil: [formatter formatBeforeXMLDeclaration: anXMLDeclarationWriter]! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 11/30/2014 04:03'!
formatter
	^ formatter! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 11/8/2016 06:11'!
formatter: aFormatter
	(formatter := aFormatter)
		ifNotNil: [formatter writer: self]! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/8/2016 07:07'!
hadActiveTagWriter
	^ hadActiveTagWriter! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/8/2016 07:14'!
hasActiveTagWriter
	^ self activeTagWriter notNil! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 4/9/2017 07:29'!
hasFormatter
	^ formatter notNil! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/22/2014 09:39'!
isCanonical
	^ isCanonical! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 4/12/2017 02:52'!
isCanonical: aBoolean
	"Should the output be canonical XML (http://www.w3.org/TR/xml-c14n)?
	(Not default)"

	self isCanonical = aBoolean
		ifTrue: [^ self].
	(isCanonical := aBoolean)
		ifTrue: [
			self
				formatter: self canonicalWriterFormatterClass new;
				lineBreak: self canonicalLineBreak;
				contentEscapes: self contentCanonicalEscapes;
				contentStringEscapes: self contentStringCanonicalEscapes;
				isEscaping: true;
				omitsRedundantXMLNSDeclarations: true]
		ifFalse: [
			self
				formatter: nil;
				lineBreak: self defaultLineBreak;
				contentEscapes: self contentDefaultEscapes;
				contentStringEscapes: self contentStringDefaultEscapes]! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/22/2014 09:41'!
isEscaping
	^ isEscaping! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 7/4/2016 05:45'!
isEscaping: aBoolean
	"can be used to disable escaping of special characters and the
	replacement of CRs/LFs in attribute/entity values and strings
	with whatever #lineBreak is set to"

	isEscaping := aBoolean! !

!XMLWriter methodsFor: 'private' stamp: 'monty 2/22/2017 11:38'!
isRedundantActiveTagWriterXMLNSPrefix: aPrefix uri: aURI
	self activeMarkupWriterAncestorsDo: [:each |
		(each isTagWriter
			and: [each hasXMLNSDeclarations])
			ifTrue: [
				each xmlnsDeclarations
					at: aPrefix
					ifPresent: [:uri | ^ uri = aURI]]].
	
	"declarations to reset the default namespace (like xmlns='') are
	considered redundant when there's no previous default namespace
	declaration being reset"
	^ aPrefix size = 0
		and: [aURI size = 0].! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 12/1/2014 06:32'!
lineBreak
	^ lineBreak! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 4/20/2016 07:58'!
lineBreak: aCharacterOrString
	"linebreak chars (CR/LF) in attribute/entity values and in args to
	the #string: and #with: messages are replaced with whatever char/string
	this is set to"

	lineBreak := aCharacterOrString asString! !

!XMLWriter methodsFor: 'private' stamp: 'monty 10/27/2014 08:19'!
markupWriterStack
	^ markupWriterStack! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/29/2016 19:19'!
newStringMarkup
	self activeMarkupWriterWhileChangingDo: [:each |
		each newStringMarkupBeingWritten]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/5/2014 04:07'!
nextPut: aCharacter
	^ stream nextPut: aCharacter! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/5/2014 04:07'!
nextPutAll: aString
	^ stream nextPutAll: aString! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 1/13/2014 23:52'!
notationDeclaration
	^ self pushNewMarkupWriter: XMLNotationDeclarationWriter! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 3/24/2015 19:27'!
omitsRedundantXMLNSDeclarations
	^ omitsRedundantXMLNSDeclarations! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 3/24/2015 19:27'!
omitsRedundantXMLNSDeclarations: aBoolean
	omitsRedundantXMLNSDeclarations := aBoolean! !

!XMLWriter methodsFor: 'initialization' stamp: 'monty 4/12/2017 02:49'!
on: aStream
	stream := aStream.
	streamStartPosition := aStream position.
	markupWriterStack := OrderedCollection new.
	lineBreak := self defaultLineBreak.
	contentEscapes := self contentDefaultEscapes.
	contentStringEscapes := self contentStringDefaultEscapes.
	isEscaping := true.
	isCanonical := false.
	outputsSelfClosingTags := true.
	omitsRedundantXMLNSDeclarations := true.
	hadActiveTagWriter := false.! !

!XMLWriter methodsFor: 'testing' stamp: 'monty 11/22/2014 09:39'!
outputsSelfClosingTags
	^ outputsSelfClosingTags! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 4/20/2016 07:52'!
outputsSelfClosingTags: aBoolean
	"can be used to disable outputting empty elements as '<empty/>'
	instead of '<empty></empty>'"

	outputsSelfClosingTags := aBoolean! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 5/8/2015 05:05'!
paramEntityRef
	^ (self pushNewMarkupWriter: XMLEntityReferenceWriter) isParameter: true! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 7/3/2016 23:59'!
paramEntityRef: aName
	"output a parameter entity reference like %name;"

	self paramEntityRef
		name: aName;
		write! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/30/2016 02:21'!
parentOfMarkupWriter: aMarkupWriter
	self markupWriterStack size to: 2 by: -1 do: [:i |
		(self markupWriterStack at: i) == aMarkupWriter
			ifTrue: [^ self markupWriterStack at: i - 1]].
	^ nil.! !

!XMLWriter methodsFor: 'deprecated' stamp: 'monty 4/16/2016 13:43'!
pcdata: aString
	self deprecated: 'use #string: instead'.
	self string: aString.! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 4/20/2016 07:59'!
pi
	"returns a new processing instruction writer for writing"

	^ self pushNewMarkupWriter: XMLPIWriter! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 4/12/2017 02:40'!
platformSpecificLineBreak
	^ PlatformSpecificLineBreak! !

!XMLWriter methodsFor: 'private' stamp: 'monty 12/13/2014 10:53'!
popActiveMarkupWriter
	^ self markupWriterStack removeLast! !

!XMLWriter methodsFor: 'accessing' stamp: 'monty 3/7/2017 13:22'!
position
	^ stream position! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 3/7/2017 13:14'!
prettyPrintingWriterFormatterClass
	^ XMLPrettyPrintingWriterFormatter! !

!XMLWriter methodsFor: 'printing' stamp: 'monty 12/5/2014 03:51'!
printOn: aStream
	aStream nextPutAll: self contents! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 3/28/2017 05:00'!
prologPrettyPrintingWriterFormatterClass
	^ XMLPrologPrettyPrintingWriterFormatter! !

!XMLWriter methodsFor: 'private' stamp: 'monty 4/12/2017 02:35'!
pushNewMarkupWriter: aMarkupWriterClass
	| newMarkupWriter |

	self activeMarkupWriterWhileChangingDo: [:each |
		each newMarkupWriterBeingPushed].
	newMarkupWriter :=
		self markupWriterStack addLast:
			(aMarkupWriterClass writer: self).
	hadActiveTagWriter
		ifFalse: [hadActiveTagWriter := newMarkupWriter isTagWriter].
	^ newMarkupWriter.! !

!XMLWriter methodsFor: 'writing - unsafe' stamp: 'monty 4/20/2016 07:59'!
raw: aString
	"writes raw string content without doing any escaping of special
	characters, unlike #string:"

	aString size > 0
		ifTrue: [
			self
				newStringMarkup;
				nextPutAll: aString]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 2/22/2017 11:41'!
resolveActiveTagWriterAttributePrefix: aPrefix
	"attributes with empty prefixes have no namespace, not even the
	default, unlike elements"
	aPrefix size > 0 "optimization"
		ifTrue: [
			(aPrefix at: 1) == $x "optimization"
				ifTrue: [
					aPrefix = 'xml'
						ifTrue: [^ self xmlNamespaceURI].
					"this prefix shouldn't be used for attributes, but we
					resolve it anyway"
					aPrefix = 'xmlns'
						ifTrue: [^ self xmlnsNamespaceURI]].

			self activeMarkupWriterWithAncestorsDo: [:each |
				(each isTagWriter
					and: [each hasXMLNSDeclarations])
					ifTrue: [
						each xmlnsDeclarations
							at: aPrefix
							ifPresent: [:uri | ^ uri]]]].

	^ ''.! !

!XMLWriter methodsFor: 'private' stamp: 'monty 5/25/2015 07:11'!
space
	"redefined for speed and portability"
	stream nextPut: SpaceCharacter! !

!XMLWriter methodsFor: 'writing - unsafe' stamp: 'monty 12/21/2015 21:54'!
startTag
	"Returns a new XMLUnsafeStartTagWriter for writing a start tag that does
	not support embedding or automatic end tag generation.

	Use the #tag* messages instead if possible."

	^ self pushNewMarkupWriter: XMLUnsafeStartTagWriter! !

!XMLWriter methodsFor: 'writing - unsafe' stamp: 'monty 12/21/2015 21:54'!
startTag: aName
	"Returns a new XMLUnsafeStartTagWriter for writing a start tag with the
	specified name that does not support embedding or automatic end tag generation.

	Use the #tag* messages instead if possible."

	^ self startTag name: aName! !

!XMLWriter methodsFor: 'writing - unsafe' stamp: 'monty 12/21/2015 21:54'!
startTag: aName attributes: aDictionaryOrArray
	"Returns a new XMLUnsafeStartTagWriter for writing a start tag with the
	specified name and attributes that does not support embedding or automatic
	end tag generation.
	
	Use the #tag* messages instead if possible."

	^ (self startTag: aName)
		attributes: aDictionaryOrArray! !

!XMLWriter methodsFor: 'private' stamp: 'monty 1/13/2014 23:14'!
stream
	^ stream! !

!XMLWriter methodsFor: 'private' stamp: 'monty 5/6/2015 16:05'!
stream: aStream
	self on: aStream! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 4/20/2016 07:52'!
string: aString
	"writes string content while '&'-escaping any special characters"

	aString size > 0
		ifTrue: [
			self
				newStringMarkup;
				writeEscapedContent: aString]! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 21:46'!
tag
	"returns a new XMLTagWriter for writing an element that supports embedding
	markup using #with: and automatic end tag generation"

	^ self pushNewMarkupWriter: XMLTagWriter! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 21:46'!
tag: aName
	"returns a new XMLTagWriter for writing an element with the specified name
	that supports embedding markup using #with: and automatic end tag generation"

	^ self tag name: aName! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 21:46'!
tag: aName attributes: aDictionaryOrArray
	"returns a new XMLTagWriter for writing an element with the specified name and
	attributes that supports embedding markup using #with: and automatic end tag
	generation"

	^ (self tag: aName)
		attributes: aDictionaryOrArray! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 14:43'!
tag: aName attributes: aDictionaryOrArray with: aStringOrCollectionOrBlock
	"writes an element with the specified name, attributes, and embedded markup"

	^ (self tag: aName)
		attributes: aDictionaryOrArray;
		with: aStringOrCollectionOrBlock! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/21/2015 14:43'!
tag: aName with: aStringOrCollectionOrBlock
	"writes an element with the specified name and embedded markup"

	^ (self tag: aName) with: aStringOrCollectionOrBlock! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 11/13/2016 21:48'!
write
	"writes any unwritten markup, including any markup writer that hasn't
	been written yet"

	self activeMarkupWriterWhileChangingDo: [:each |
		each hasStartedWriting
			ifFalse: [each write]]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:11'!
write: aString escapedWith: anEscapeTable
	| tableSize |

	tableSize := anEscapeTable size.
	"to:do: for performance"
	1 to: aString size do: [:i | | nextChar asciiValue |
		((asciiValue := (nextChar := aString at: i) asciiValue) > 0
			and: [asciiValue <= tableSize
				and: [(anEscapeTable at: asciiValue) notNil]])
			ifTrue: [| replacement |
				(replacement := anEscapeTable at: asciiValue) == #LineBreak
					ifTrue: [stream nextPutAll: self lineBreak]
					ifFalse: [
						formatter
							ifNil: [stream nextPutAll: replacement]
							ifNotNil: [
								formatter formatBeforeCharacterEscape:
									replacement.
								stream nextPutAll: replacement.
								formatter formatAfterCharacterEscape:
									replacement]]]
			ifFalse: [stream nextPut: nextChar]].! !

!XMLWriter methodsFor: 'private' stamp: 'monty 4/12/2017 23:20'!
writeAttributeName: aName unescapedValue: aValue
	self
		space;
		writeStringName: aName;
		nextPut: $=;
		writeUnescapedString: aValue! !

!XMLWriter methodsFor: 'private' stamp: 'monty 4/12/2017 23:20'!
writeAttributeName: aName value: aValue
	self
		space;
		writeStringName: aName;
		nextPut: $=;
		writeEscapedContentString: aValue! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/30/2016 19:55'!
writeAttributes: anAttributeCollection
	self isCanonical
		ifTrue: [anAttributeCollection writeCanonicalAttributesOn: self]
		ifFalse: [anAttributeCollection writeAttributesOn: self]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:27'!
writeEscapedContent: aString
	self isEscaping
		ifTrue: [
			self
				write: aString
				escapedWith: contentEscapes]
		ifFalse: [stream nextPutAll: aString]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 4/9/2017 07:27'!
writeEscapedContentString: aString
	stream nextPut: $".
	formatter
		ifNotNil: [formatter formatBeforeQuotedStringValue: aString].
	self isEscaping
		ifTrue: [
			self
				write: aString
				escapedWith: contentStringEscapes]
		ifFalse: [stream nextPutAll: aString].
	formatter
		ifNotNil: [formatter formatAfterQuotedStringValue: aString].
	stream nextPut: $".! !

!XMLWriter methodsFor: 'private' stamp: 'monty 4/12/2017 02:55'!
writeEscapedEntityString: aString
	stream nextPut: $".
	formatter
		ifNotNil: [formatter formatBeforeQuotedStringValue: aString].
	self isEscaping
		ifTrue: [
			self
				write: aString
				escapedWith: self entityValueEscapes]
		ifFalse: [stream nextPutAll: aString].
	formatter
		ifNotNil: [formatter formatAfterQuotedStringValue: aString].
	stream nextPut: $".! !

!XMLWriter methodsFor: 'private' stamp: 'monty 4/3/2017 23:02'!
writeName: aName
	formatter
		ifNil: [stream nextPutAll: aName]
		ifNotNil: [
			formatter formatBeforeName: aName.
			stream nextPutAll: aName.
			formatter formatAfterName: aName]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 4/9/2017 08:11'!
writeStringName: aName
	formatter
		ifNil: [stream nextPutAll: aName]
		ifNotNil: [
			formatter formatBeforeQuotedStringName: aName.
			stream nextPutAll: aName.
			formatter formatAfterQuotedStringName: aName]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 4/7/2017 02:07'!
writeUnescapedString: aString
	self
		writeUnescapedString: aString
		quotedBy:
			((aString includes: $")
				ifTrue: [$']
				ifFalse: [$"])! !

!XMLWriter methodsFor: 'private' stamp: 'monty 4/8/2017 00:32'!
writeUnescapedString: aString quotedBy: aQuote
	stream nextPut: aQuote.
	formatter
		ifNotNil: [
			formatter formatBeforeQuotedStringValue: aString].
	1 to: aString size do: [:i | | nextChar |
		"delimiters are skipped because they can't be escaped here"
		(nextChar := aString at: i) == aQuote
			ifFalse: [stream nextPut: nextChar]].
	formatter
		ifNotNil: [
			formatter formatAfterQuotedStringValue: aString].
	stream nextPut: aQuote.! !

!XMLWriter methodsFor: 'private' stamp: 'monty 11/13/2016 21:48'!
writeUnwrittenMarkupWritersEmbeddedIn: aMarkupWriter
	self activeMarkupWriterWhileChangingDo: [:each |
		each == aMarkupWriter
			ifTrue: [^ self].
		each hasStartedWriting
			ifFalse: [each write]]! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 4/20/2016 07:54'!
writeWith: aBlock
	"evaluates aBlock with the receiver as an optional argument and
	automatically sends #write after"

	aBlock cull: self.
	self write.! !

!XMLWriter methodsFor: 'private' stamp: 'monty 4/12/2017 23:12'!
writeXMLNS: aPrefix
	self writeStringName:
		(aPrefix isEmpty
			ifTrue: ['xmlns']
			ifFalse: ['xmlns:', aPrefix])! !

!XMLWriter methodsFor: 'private' stamp: 'monty 4/12/2017 23:13'!
writeXMLNS: aPrefix uri: aURI
	(self omitsRedundantXMLNSDeclarations
		and: [
			self
				isRedundantActiveTagWriterXMLNSPrefix: aPrefix
				uri: aURI])
		ifFalse: [
			self
				space;
				writeXMLNS: aPrefix;
				nextPut: $=;
				writeEscapedContentString: aURI]! !

!XMLWriter methodsFor: 'private' stamp: 'monty 2/26/2017 09:15'!
writeXMLNSDeclarations: aDictionary
	self isCanonical
		ifTrue: [
			aDictionary keys
				sort;
				do: [:each |
					self
						writeXMLNS: each
						uri: (aDictionary at: each)]]
		ifFalse: [
			aDictionary keysDo: [:each |
				self
					writeXMLNS: each
					uri: (aDictionary at: each)]]! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 4/20/2016 07:53'!
xml
	"an abbreviation"

	^ self xmlDeclaration! !

!XMLWriter methodsFor: 'writing' stamp: 'monty 12/20/2015 23:09'!
xmlDeclaration
	^ self pushNewMarkupWriter: XMLXMLDeclarationWriter! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 12/30/2016 23:31'!
xmlNamespaceURI
	^ 'http://www.w3.org/XML/1998/namespace'! !

!XMLWriter methodsFor: 'defaults' stamp: 'monty 12/30/2016 23:31'!
xmlnsNamespaceURI
	^ 'http://www.w3.org/2000/xmlns/'! !

Object subclass: #XMLWriterFormatter
	instanceVariableNames: 'writer indentString currentIndentLevel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLWriterFormatter commentStamp: 'monty 5/8/2015 05:00' prior: 0!
An abstract class for formatters used to decorate markup writers. Override the "formatting" messages.!

XMLWriterFormatter subclass: #XMLCanonicalWriterFormatter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLCanonicalWriterFormatter commentStamp: 'monty 3/31/2017 00:31' prior: 0!
A formatter that adds LF linebreaks around canonical markup.!

!XMLCanonicalWriterFormatter methodsFor: 'formatting' stamp: 'monty 11/8/2016 07:27'!
formatAfterComment: aCommentWriter
	self writer hadActiveTagWriter
		ifFalse: [self writeLineBreak]! !

!XMLCanonicalWriterFormatter methodsFor: 'formatting' stamp: 'monty 11/8/2016 07:26'!
formatAfterPI: aPIWriter
	self writer hadActiveTagWriter
		ifFalse: [self writeLineBreak]! !

!XMLCanonicalWriterFormatter methodsFor: 'formatting' stamp: 'monty 11/8/2016 07:26'!
formatBeforeComment: aCommentWriter
	(self writer hadActiveTagWriter
		and: [self writer hasActiveTagWriter not])
		ifTrue: [self writeLineBreak]! !

!XMLCanonicalWriterFormatter methodsFor: 'formatting' stamp: 'monty 11/8/2016 07:26'!
formatBeforePI: aPIWriter
	(self writer hadActiveTagWriter
		and: [self writer hasActiveTagWriter not])
		ifTrue: [self writeLineBreak]! !

XMLWriterFormatter subclass: #XMLPrologPrettyPrintingWriterFormatter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLPrologPrettyPrintingWriterFormatter commentStamp: 'monty 3/28/2017 04:22' prior: 0!
A formatter that inserts whitespace formatting to make the prolog XML output nicer and more readable, but doesn't format content XML.!

XMLPrologPrettyPrintingWriterFormatter subclass: #XMLPrettyPrintingWriterFormatter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Writer-Core'!

!XMLPrettyPrintingWriterFormatter commentStamp: 'monty 7/3/2016 23:29' prior: 0!
A formatter that inserts whitespace formatting to make XML output nicer and more readable.!

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 3/28/2017 04:10'!
formatAfterDoctypeDeclaration: aDoctypeDeclarationWriter
	self writeLineBreak! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:33'!
formatAfterStartTag: aTagWriter
	self indent! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:39'!
formatBeforeComment: aCommentWriter
	aCommentWriter isEmbeddedWithStringMarkup
		ifFalse: [self writeLineBreak]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:33'!
formatBeforeEndTag: aTagWriter
	aTagWriter hasEmbeddedMarkup
		ifTrue: [
			self outdent.
			aTagWriter hasEmbeddedStringMarkup
				ifFalse: [self writeLineBreak]]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:12'!
formatBeforePI: aPIWriter
	aPIWriter isEmbeddedWithStringMarkup
		ifFalse: [self writeLineBreak]! !

!XMLPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:50'!
formatBeforeStartTag: aTagWriter
	aTagWriter isEmbeddedWithStringMarkup
		ifFalse: [self writeLineBreak]! !

!XMLPrologPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 3/28/2017 03:32'!
formatAfterInternalSubset: aDoctypeDeclarationWriter
	aDoctypeDeclarationWriter hasEmbeddedMarkup
		ifTrue: [
			self
				outdent;
				writeLineBreak]! !

!XMLPrologPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 3/28/2017 04:11'!
formatBeforeComment: aCommentWriter
	aCommentWriter isEmbeddedInTagWriter
		ifFalse: [self writeLineBreak]! !

!XMLPrologPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 3/28/2017 03:31'!
formatBeforeDeclaration: aDeclarationWriter
	self writeLineBreak! !

!XMLPrologPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 3/28/2017 03:31'!
formatBeforeDoctypeDeclaration: aDoctypeDeclarationWriter
	self writeLineBreak! !

!XMLPrologPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 3/28/2017 03:31'!
formatBeforeEntityReference: anEntityReferenceWriter
	anEntityReferenceWriter isParameter
		ifTrue: [self writeLineBreak]! !

!XMLPrologPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 3/28/2017 03:31'!
formatBeforeInternalSubset: aDoctypeDeclarationWriter
	self indent! !

!XMLPrologPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 3/28/2017 04:11'!
formatBeforePI: aPIWriter
	aPIWriter isEmbeddedInTagWriter
		ifFalse: [self writeLineBreak]! !

!XMLPrologPrettyPrintingWriterFormatter methodsFor: 'formatting' stamp: 'monty 3/28/2017 04:12'!
formatBeforeStartTag: aTagWriter
	aTagWriter isEmbeddedInTagWriter
		ifFalse: [self writeLineBreak]! !

!XMLWriterFormatter class methodsFor: 'accessing' stamp: 'monty 5/25/2015 06:54'!
defaultIndentLevel
	^ 0! !

!XMLWriterFormatter class methodsFor: 'accessing' stamp: 'monty 5/25/2015 06:54'!
defaultIndentString
	^ '    '! !

!XMLWriterFormatter class methodsFor: 'instance creation' stamp: 'monty 9/20/2015 23:24'!
writer: aWriter
	^ self new writer: aWriter! !

!XMLWriterFormatter methodsFor: 'private' stamp: 'monty 11/30/2014 03:25'!
currentIndentLevel
	^ currentIndentLevel! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 9/20/2015 02:55'!
formatAfterCData: aCDataWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 4/9/2017 02:25'!
formatAfterCharacterEscape: aCharacterEscape
	"sent after outputting an escape sequence for some character"! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:09'!
formatAfterComment: aCommentWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:14'!
formatAfterDeclaration: aDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:45'!
formatAfterDoctypeDeclaration: aDoctypeDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:06'!
formatAfterEndTag: aTagWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:16'!
formatAfterEntityReference: anEntityReferenceWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:42'!
formatAfterInternalSubset: aDoctypeDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 4/3/2017 23:03'!
formatAfterName: aName! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:12'!
formatAfterPI: aPIWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 4/4/2017 22:27'!
formatAfterQuotedStringName: aName! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 9/18/2015 18:53'!
formatAfterQuotedStringValue: aString! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:37'!
formatAfterStartTag: aTagWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:11'!
formatAfterXMLDeclaration: anXMLDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 9/20/2015 02:55'!
formatBeforeCData: aCDataWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 4/9/2017 02:26'!
formatBeforeCharacterEscape: aCharacterEscape
	"sent before outputting an escape sequence for some character"! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:38'!
formatBeforeComment: aCommentWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:40'!
formatBeforeDeclaration: aDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:44'!
formatBeforeDoctypeDeclaration: aDoctypeDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:38'!
formatBeforeEndTag: aTagWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/2/2014 00:35'!
formatBeforeEntityReference: anEntityReferenceWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:46'!
formatBeforeInternalSubset: aDoctypeDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 4/3/2017 23:03'!
formatBeforeName: aName! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 5/8/2015 04:12'!
formatBeforePI: aPIWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 4/4/2017 22:28'!
formatBeforeQuotedStringName: aName! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 9/18/2015 18:53'!
formatBeforeQuotedStringValue: aString! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 12/1/2014 23:38'!
formatBeforeStartTag: aTagWriter! !

!XMLWriterFormatter methodsFor: 'formatting' stamp: 'monty 3/24/2015 19:34'!
formatBeforeXMLDeclaration: anXMLDeclarationWriter! !

!XMLWriterFormatter methodsFor: 'indenting' stamp: 'monty 11/30/2014 03:25'!
indent
	currentIndentLevel := self currentIndentLevel + 1! !

!XMLWriterFormatter methodsFor: 'accessing' stamp: 'monty 11/30/2014 03:16'!
indentString
	^ indentString! !

!XMLWriterFormatter methodsFor: 'accessing' stamp: 'monty 11/30/2014 03:16'!
indentString: aString
	currentIndentLevel := 0.
	indentString := aString.! !

!XMLWriterFormatter methodsFor: 'initialization' stamp: 'monty 9/20/2015 23:24'!
initialize
	super initialize.

	currentIndentLevel := self class defaultIndentLevel.
	indentString := self class defaultIndentString.! !

!XMLWriterFormatter methodsFor: 'indenting' stamp: 'monty 11/30/2014 03:25'!
outdent
	currentIndentLevel := self currentIndentLevel - 1! !

!XMLWriterFormatter methodsFor: 'writing' stamp: 'monty 5/18/2017 20:27'!
writeIndent
	"use #to:do: over #timesRepeat: for speed"
	1 to: self currentIndentLevel do: [:i |
		self writer nextPutAll: self indentString]! !

!XMLWriterFormatter methodsFor: 'writing' stamp: 'monty 4/12/2017 02:33'!
writeLineBreak
	self writer atBeginning
		ifFalse: [
			self writer nextPutAll: self writer lineBreak.
			(self currentIndentLevel > 0
				and: [self indentString size > 0])
				ifTrue: [self writeIndent]]! !

!XMLWriterFormatter methodsFor: 'accessing' stamp: 'monty 11/30/2014 02:44'!
writer
	^ writer! !

!XMLWriterFormatter methodsFor: 'accessing' stamp: 'monty 9/21/2015 01:01'!
writer: aWriter
	writer := aWriter! !

!BlockClosure methodsFor: '*XML-Writer-Core' stamp: 'monty 4/20/2016 07:47'!
embedInXMLMarkupWriter: aMarkupWriter
	self cull: aMarkupWriter writer! !

XMLWriter initialize!
